package %PACKAGE%;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.PointF;
import android.graphics.RectF;

public class Item {
	
	// position
	private float x;
	private float y;	
	
	private PointF speed;	
	PointF size;	
	private Bitmap image;
	private State state;
	private Paint paint;
		
	private static final float GROW_TO = 0.5f; // Item grows by 50% of its original size while in the dying state
	private static final long TIMER_DYING_STATE = 100; // milliseconds
	private long mElapsedTimeInDyingState = 0; // milliseconds
	
	private int streakCounter = 0;
	
	// constructor
	public Item(PointF position, PointF speed, Bitmap image) {		
		this.x = position.x;
		this.y = position.y;	
		this.speed = speed;				
		this.size = new PointF(image.getWidth(), image.getHeight());		
		this.image = image;		
		this.state = State.ALIVE;
		this.paint = new Paint();
		this.paint.setColor(Color.RED);
		paint.setStyle(Style.STROKE);
		paint.setAntiAlias(true);
		paint.setFilterBitmap(true);
	}	
	
	public void update(long elapsedTime) {		
		this.x += this.speed.x;
		this.y += this.speed.y;
		
		if (this.state == State.DYING) mElapsedTimeInDyingState += elapsedTime;	
		if (mElapsedTimeInDyingState >= TIMER_DYING_STATE)  {
			this.state = State.DEAD;
			mElapsedTimeInDyingState = 0;
		}		
	}
	
	public void update(long elapsedTime, float xOffsetRatio) {	
		this.x += xOffsetRatio * this.size.x;		
		this.update(elapsedTime);
	}
	
	public void draw(Canvas c){		
		if (this.state == State.ALIVE) {
			c.drawBitmap(this.image, this.x, this.y, null);
			//c.drawRect(this.x, this.y, this.x + this.size.x, this.y + this.size.y, paint);
		}
		else if (this.state == State.DYING) {
			RectF r = this.getScaledRect(1 + GROW_TO * ((float)mElapsedTimeInDyingState / TIMER_DYING_STATE));
			c.drawBitmap(this.image, null, r, paint);
			//c.drawRect(r, paint);
		}
	}
	
	public void setPosition(PointF point) {		
		this.x = point.x;
		this.y = point.y;
	}
	
	public void setPosition(float x, float y) {		
		this.x = x;
		this.y = y;
	}	
	
	public void setSpeed(PointF speed) {		
		this.speed = speed;
	}
	
	public void setImage(Bitmap newImage) {		
		this.image = newImage;		
		this.size.x = newImage.getWidth();
		this.size.y = newImage.getHeight();
	}
	
	public PointF getPos() {		
		return new PointF(this.x, this.y);
	}
	
	public void setAliveState() {
		this.state = State.ALIVE;
	}
	
	public void setDyingState(int streak) {		
		this.state = State.DYING;
		this.streakCounter = streak;
	}
	
	public boolean isAlive(){
		return this.state != State.DEAD;
	}
	
	public State getState() {
		return this.state;		
	}
	
	private RectF getScaledRect(float ratio) {		
		
		float dx =  this.size.x / 2 * (ratio - 1);
		float dy =  this.size.y / 2 * (ratio - 1);
		
		return new RectF(	this.x - dx,
							this.y - dy, 
							this.x - dx + this.size.x * ratio, 
							this.y - dy + this.size.y * ratio);
	}
	
	public float getWidth() {
		return this.size.x;	
	}
	
	public float getHeight() {
		return this.size.y;	
	}
	
	public Bitmap getBitmap() {
		return this.image;	
	}
	
	public int getStreak() {
		return this.streakCounter;
	}
}